package com.mystudy.net_multi2;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;
import java.util.Iterator;

/* === 멀티채팅 서버측 기능 ===
 1. 다중 접속 가능하게 처리
 2. 접속자가 보낸 메세지를 읽을 수 있는 읽기 전용 쓰레드 생성
 ------------
 3. 접속자 명단 가지고 있기
 4. 받은 메시지를 접속자 모두에게 전송하는 기능
 */

public class TCPServerMultiChat {

	private HashMap<String, DataOutputStream> clients; // 접속자 명단
	
	public TCPServerMultiChat() {
		clients = new HashMap<String, DataOutputStream>();
	}
	
	public static void main(String[] args) {
		System.out.println("=== main() 시작 ===");

		new TCPServerMultiChat().serverStart();
		
		System.out.println("=== main() 끝 ===");
	}

	// 서버소켓 객체 생성하고 대기하다가 
	// 클라이언트가 접속하면 메시지를 읽기 위한 읽기전용 Thread 생성
	public void serverStart() {
		ServerSocket serverSocket = null;
		
		try {
			serverSocket = new ServerSocket(10000);
			System.out.println(">>> 서버가 시작되었습니다");
			System.out.println("IP번호:포트번호 -"
					+ InetAddress.getLocalHost().getHostAddress()
					+ ":" + serverSocket.getLocalPort());
			
			
			while (true) {
				Socket socket = serverSocket.accept();
				System.out.println("["+ socket.getInetAddress()
				+ ":" + socket.getPort() + "]");
				
				// 접속된 클라이언트에 대한 처리 : Thread 생성
				ServerReceiver receiver = new ServerReceiver(socket);
				receiver.start();
			}
			
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	// ServerReceiver : 전달받은 Socket을 이용해서 메시지 받고, 전체에게 전달
	// 생성될 때 : 접속자명단 (필드 clients)에 등록(추가)
	// 종료될 때 : 접속자명단 (필드 clients)에 제외(삭제)
	private class ServerReceiver extends Thread {
		Socket socket;
		DataInputStream in;
		DataOutputStream out;
		String ip;
		String name;
		
		public ServerReceiver(Socket socket) {
			this.socket = socket;
			try {
				in = new DataInputStream(socket.getInputStream());
				out = new DataOutputStream(socket.getOutputStream());
				
				ip = socket.getInetAddress().getHostAddress();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		@Override
		public void run() {
			// 생성될때
			try {
				// 클라이언트로부터 받은 첫 메세지 : 별칭(아이디)
				name = in.readUTF();
				System.out.println("접속자 >> " + name);
				
				// 전체에게 접속한 사람 알림
				sendToAll("#" + name + "님이 입장했습니다.");
				
				// 접속자 정보를 명단(clients)에 등록
				clients.put(name, out);
				System.out.println(":: 현재 접속자 수 : " + clients.size());
				System.out.println("접속자 : " + clients);
				
				// 메세지를 읽고, 전체에게 전달(반복)
				while (true) {
					String msg = in.readUTF();
					System.out.println(name + ">" + msg); // 서버화면에 출력
					sendToAll(msg); // 접속자 전원에게 전송
				}
				
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				// 클라이언트 접속 종료시 처리
				// 종료할때 명단(clients)에서 삭제
				clients.remove(name);
				String outMsg = "<" + name + "> 님이 나갔습니다";
				sendToAll(outMsg);
				System.out.println(outMsg);
			}
			
			
			// 종료될때
		}

		// 접속자 모두에게 전달받은 메시지 전송
		private void sendToAll(String msg) {

			// key값 가져오기 (iterator 사용)
			Iterator<String> iterator = clients.keySet().iterator();
			
			while (iterator.hasNext()) {
				String key = iterator.next();
				// 자신이 보낸 메세지 걸러내기
				DataOutputStream out = clients.get(key);
				
				try {
					out.writeUTF(msg);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			
		}
	}
	
}
